@@ -2,8 +2,10 @@ import requests
 import base64
 import datetime
 import traceback
+import logging
 import json
 import os
+from pynexusic import utils
 
 ############################# Maximo REST API Class ############################
 class MAXIMO_REST():
@@ -23,7 +25,8 @@ class MAXIMO_REST():
     def __init__(self, base_url, username=None, password=None, api_key=None,
                  authentication_type='LDAP BASIC',
                  max_attempts=1, timeout=None, mif=False,
-                 verbose=False, verify=True):
+                 verbose=False, verify=True,
+                 logger=None):
         """
 
             :param maximo_url: (``string``) - Maximo URL
@@ -62,6 +65,7 @@ class MAXIMO_REST():
 
         self.verbose = verbose
         self.verify = verify
+        self.logger = logger
         self.api_key = self.generate_base64(self.username + ':' + self.password)
         self.session = None
         self.authenticate()
@@ -113,17 +117,14 @@ class MAXIMO_REST():
             :returns: None
         """
 
-        if self.verbose or verbose:
-            print('Authenticating with Maximo...')
+        if verbose:
+            utils.print_log('Authenticating with Maximo...', logger=self.logger, logLvl=logging.INFO)
 
-        if self.authentication_type == 'LDAP BASIC':
-            url = self.base_url + '/maximo' + self.url_suffix + '/j_security_check?j_username=' + \
-                  self.username + '&j_password=' + self.password
+        url = self.base_url + '/maximo' + self.url_suffix + '/j_security_check?j_username=' + \
+              self.username + '&j_password=' + self.password
 
-            self.session = requests.Session()
-            self.session.get(url, verify=self.verify)
-        else:
-            raise Exception(self.authentication_type + ' authentication type not supported')
+        self.session = requests.Session()
+        self.session.get(url, verify=self.verify)
 
     ######################################################################################
     def add_param_to_url(self, url, param, value):
@@ -218,8 +219,8 @@ class MAXIMO_REST():
             :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Getting ' + str(tableName) + ' table...')
+        if verbose:
+            utils.print_log('Getting ' + str(tableName) + ' table...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /oslc/ part to the baseURI
         url = self.base_url + '/maximo' + self.url_suffix + '/oslc/os/' + tableName
@@ -326,8 +327,8 @@ class MAXIMO_REST():
             :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Inserting new record in ' + str(tableName) + '...')
+        if verbose:
+            utils.print_log('Inserting new record in ' + str(tableName) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /oslc/ part to the baseURI
         url = self.base_url + '/maximo' + self.url_suffix + '/oslc/os/' + tableName
@@ -357,8 +358,8 @@ class MAXIMO_REST():
             :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code
         """
 
-        if self.verbose or verbose:
-            print('Updating ' + str(tableName) + ' ' + str(restID) + '...')
+        if verbose:
+            utils.print_log('Updating ' + str(tableName) + ' ' + str(restID) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /oslc/ part to the baseURI
         if use_oslc:
@@ -404,8 +405,8 @@ class MAXIMO_REST():
         :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Getting ' + str(tableName) + ' table...')
+        if verbose:
+            utils.print_log('Getting ' + str(tableName) + ' table...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /maxrest/ part to the baseURI
         url = self.base_url + '/maxrest' + self.url_suffix + '/rest/mbo/' + tableName
@@ -465,8 +466,8 @@ class MAXIMO_REST():
             :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Inserting new record in ' + str(tableName) + '...')
+        if verbose:
+            utils.print_log('Inserting new record in ' + str(tableName) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /maxrest/ part to the baseURI
         url = self.base_url + '/maxrest' + self.url_suffix + '/rest/mbo/' + tableName
@@ -494,8 +495,8 @@ class MAXIMO_REST():
             :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Updating ' + str(tableName) + ' ' + str(tableID) + '...')
+        if verbose:
+            utils.print_log('Updating ' + str(tableName) + ' ' + str(tableID) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /maxrest/ part to the baseURI
         url = self.base_url + '/maxrest' + self.url_suffix + '/rest/mbo/' + tableName + '/' + str(tableID)
@@ -524,8 +525,9 @@ class MAXIMO_REST():
             :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Executing method ' + str(method) + ' on table ' + str(tableName) + ' on record ' + str(tableID) + '...')
+        if verbose:
+            utils.print_log('Executing method ' + str(method) + ' on table ' + str(tableName) + ' on record ' + str(tableID) + '...',
+                            logger=self.logger, logLvl=logging.INFO)
 
         # Adding /maxrest/ part to the baseURI
         url = self.base_url + '/maxrest' + self.url_suffix + '/rest/mbo/' + tableName + '/' + str(tableID)
