@@ -2,7 +2,9 @@ import requests
 import base64
 import datetime
 import traceback
+import logging
 import json
+from pynexusic import utils
 
 ############################# NEXUS REST API V2 Functions ############################
 class NEXUSIC_REST():
@@ -18,9 +20,7 @@ class NEXUSIC_REST():
     NEXUS IC Documentation:
     ----------------------
         The NEXUS IC REST API documentation can be found in the below link:
-        https://docs.nexusic.com/6.6/ic-web.rest.v2.html
-
-        A specific NEXUS IC version can be specified in the above link by changing **6.6** to the desired NEXUS IC version*
+        https://docs.nexusic.com
     """
 
     _error_msgs = ['An existing connection was forcibly closed by the remote host',
@@ -28,7 +28,8 @@ class NEXUSIC_REST():
 
     def __init__(self, icweb_uri, authentication_type='APIKEY',
                  username=None, password=None, api_key=None,
-                 max_attempts=1, timeout=None, verbose=False, verify=True):
+                 max_attempts=1, timeout=None, verbose=False, verify=True,
+                 logger=None):
         """
 
             :param icweb_uri: (``string``) - IC-Web URL.
@@ -60,6 +61,7 @@ class NEXUSIC_REST():
         self.timeout = timeout   # TODO: To be added to the REST calls
         self.verbose = verbose
         self.verify = verify
+        self.logger = logger
 
         assert authentication_type in ['APIKEY', 'BASIC'], 'Incorrect authentication type'
 
@@ -156,13 +158,17 @@ class NEXUSIC_REST():
             :return: (``tuple`` - dict or string, string) - response (json, raw or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Authenticating with NEXUS IC...')
+        if verbose:
+            utils.print_log('Authenticating with NEXUS IC...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
 
         uri = baseURI + '/security/login'
+
+        if self.logger != None and self.logger.level > 10:
+            utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
         res = requests.get(uri, headers={'Authorization': self.authentication_type + ' ' + self.key_64},
                            verify=self.verify)
 
@@ -170,14 +176,17 @@ class NEXUSIC_REST():
 
     ######################################################################################
     def getVersion(self, current_attempt=1, verbose=False):
-        if self.verbose or verbose:
-            print('Getting NEXUS IC version...')
+        if verbose:
+            utils.print_log('Getting NEXUS IC version...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
 
         uri = baseURI + '/version' + '?hash=' + self.hash
 
+        if self.logger != None and self.logger.level > 10:
+            utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
         try:
             res = requests.get(uri, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Get version error ')
@@ -208,8 +217,8 @@ class NEXUSIC_REST():
             :return: (``tuple`` - dict or string, string) - response (json, raw or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Getting ' + str(tableDBName) + '...')
+        if verbose:
+            utils.print_log('Getting ' + str(tableDBName) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
@@ -230,8 +239,16 @@ class NEXUSIC_REST():
         uri = self.addParamToURI(uri, 'hash', self.hash)
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             if xFilter != None:
-                res = requests.get(uri, headers={'X-NEXUS-Filter': xFilter}, verify=self.verify)
+                headers = {'X-NEXUS-Filter': xFilter}
+
+                if self.logger != None and self.logger.level > 10:
+                    utils.print_log(headers, logger=self.logger, logLvl=logging.DEBUG)
+
+                res = requests.get(uri, headers=headers, verify=self.verify)
             else:
                 res = requests.get(uri, verify=self.verify)
 
@@ -241,6 +258,9 @@ class NEXUSIC_REST():
             uri = uri.replace('startRow=' + str(startRow), 'startRow=' + str(pageSize))
             startRow = pageSize
             while len(result['rows']) < result['totalRows']:
+                if self.logger != None and self.logger.level > 10:
+                    utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
                 if xFilter != None:
                     res = requests.get(uri, headers={'X-NEXUS-Filter': xFilter}, verify=self.verify)
                 else:
@@ -268,8 +288,8 @@ class NEXUSIC_REST():
 
     ######################################################################################
     def deleteRecord(self, tableDBName, keyValue, current_attempt=1, verbose=False):
-        if self.verbose or verbose:
-            print('Deleting from ' + str(tableDBName) + '...')
+        if verbose:
+            utils.print_log('Deleting from ' + str(tableDBName) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
@@ -277,6 +297,9 @@ class NEXUSIC_REST():
         uri = baseURI + '/bo/' + tableDBName + '/' + str(keyValue) + '/' + '?hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.delete(uri, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Delete ' + tableDBName + ' table error ')
             return result, result_code
@@ -293,8 +316,8 @@ class NEXUSIC_REST():
 
     ######################################################################################
     def getMultimedia(self, rd_id, current_attempt=1, verbose=False):
-        if self.verbose or verbose:
-            print('Getting multimedia: ' + str(rd_id) + '...')
+        if verbose:
+            utils.print_log('Getting multimedia: ' + str(rd_id) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
@@ -303,6 +326,9 @@ class NEXUSIC_REST():
         uri = baseURI + '/bo/' + 'Repository_Data' + '/' + xFilter + '/' + '?hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.get(uri, stream=True, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Get multimeida error ', raw=True)
             return result, result_code
@@ -324,8 +350,8 @@ class NEXUSIC_REST():
         if not(int(self.version[1]) >= 6):
             return 'This function is not supported in the current NEXUS IC version', 404
 
-        if self.verbose or verbose:
-            print('Generating NEXUS IC dashboard...')
+        if verbose:
+            utils.print_log('Generating NEXUS IC dashboard...', logger=self.logger, logLvl=logging.INFO)
 
         # Get RT_ID
         xFilter = '{"where": [{"field": "Name", "value": "' + dashboard_Name + '"}]}'
@@ -342,6 +368,9 @@ class NEXUSIC_REST():
         uri = baseURI + '/dashboard/' + str(rt_id) + '/' + '?hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.get(uri, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Get ' + dashboard_Name + ' error ')
             return result, result_code
@@ -359,8 +388,8 @@ class NEXUSIC_REST():
     ######################################################################################
     def generateReport(self, report_name, recipient, format='XLSX', current_attempt=1,
                        verbose=False):
-        if self.verbose or verbose:
-            print('Generating NEXUS IC report...')
+        if verbose:
+            utils.print_log('Generating NEXUS IC report...', logger=self.logger, logLvl=logging.INFO)
 
         # Get RT_ID
         xFilter = '{"where": [{"field": "Name", "value": "' + report_name + '"}]}'
@@ -379,6 +408,9 @@ class NEXUSIC_REST():
         uri += '?key=' + str(rt_id) + '&format=' + format + '&recipient=' + recipient + '&hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.post(uri, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Generate ' + report_name + ' report error ')
             return result, result_code
@@ -414,16 +446,16 @@ class NEXUSIC_REST():
         if not (int(self.version[1]) >= 7):
             return 'This function is not supported in the current NEXUS IC version', 404
 
-        if self.verbose or verbose:
-            print('Generating NEXUS IC report...')
+        if verbose:
+            utils.print_log('Generating NEXUS IC report...', logger=self.logger, logLvl=logging.INFO)
 
         rt_id = report_id
         
         # Get RT_ID
         if not rt_id:
-            rt_id = self.get_rt_id(report_name)
-            if self.verbose or verbose:
-                print('Determined Report ID = {}'.format(str(rt_id)))
+            rt_id = self.get_rt_id(report_name, verbose=verbose)
+            if verbose:
+                utils.print_log('Determined Report ID = {}'.format(str(rt_id)), logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
@@ -435,6 +467,9 @@ class NEXUSIC_REST():
                                 'parameters': parameters
                                 })
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.post(uri, data=json_dict)
             result, result_code = self.validate_and_return_response(res, 'Generate ' + (report_name if report_name else "") + str((report_id if report_id else "")) + ' report error ')
             return result, result_code
@@ -466,14 +501,17 @@ class NEXUSIC_REST():
         if not (int(self.version[1]) >= 7):
             return 'This function is not supported in the current NEXUS IC version', 404
         
-        if self.verbose or verbose:
-            print('Getting NEXUS IC job status...')
+        if verbose:
+            utils.print_log('Getting NEXUS IC job status...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
         uri = baseURI + '/jobStatus' + '?hash=' + self.hash + '&id=' + str(job_id)
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.get(uri)
             result, result_code = self.validate_and_return_response(res, 'Get Job Status of Job ' + str(job_id) + ' error ')
             return result, result_code
@@ -504,14 +542,17 @@ class NEXUSIC_REST():
         if not (int(self.version[1]) >= 7): 
             return 'This function is not supported in the current NEXUS IC version', 404
 
-        if self.verbose or verbose:
-            print('Getting NEXUS IC job content...')
+        if verbose:
+            utils.print_log('Getting NEXUS IC job content...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
         uri = baseURI + '/jobContent' + '?hash=' + self.hash + '&id=' + str(job_id)
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.get(uri, stream=True)
             result, result_code = self.validate_and_return_response(res, 'Get Job Content of Job ' + str(job_id) + ' error ', raw=True)
             return result, result_code
@@ -529,8 +570,8 @@ class NEXUSIC_REST():
     ######################################################################################
     def execFunction(self, functionName, parameters=None, current_attempt=1,
                      verbose=False):
-        if self.verbose or verbose:
-            print('Executing ' + str(functionName) + '...')
+        if verbose:
+            utils.print_log('Executing ' + str(functionName) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
@@ -539,6 +580,10 @@ class NEXUSIC_REST():
         uri += '/' + '?hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+                utils.print_log(str(parameters), logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.post(uri, json=parameters, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Execute function ' + functionName + ' error ')
             return result, result_code
@@ -556,8 +601,8 @@ class NEXUSIC_REST():
 
     ######################################################################################
     def execUpdate(self, tableDBName, tableID, body, current_attempt=1, verbose=False):
-        if self.verbose or verbose:
-            print('Updating ' + str(tableDBName) + '...')
+        if verbose:
+            utils.print_log('Updating ' + str(tableDBName) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
@@ -566,6 +611,10 @@ class NEXUSIC_REST():
         uri += '?hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+                utils.print_log(str(body), logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.post(uri, json=body, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Updating ' + tableDBName + ' error ')
             return result, result_code
@@ -583,8 +632,8 @@ class NEXUSIC_REST():
 
     ######################################################################################
     def createNewRecord(self, tableDBName, body, key_value=0, current_attempt=1, verbose=False):
-        if self.verbose or verbose:
-            print('Creating new record in ' + str(tableDBName) + '...')
+        if verbose:
+            utils.print_log('Creating new record in ' + str(tableDBName) + '...', logger=self.logger, logLvl=logging.INFO)
 
         # Adding /data/icweb.dll part to the baseURI
         baseURI = self.icweb_uri + '/data/icweb.dll'
@@ -593,6 +642,10 @@ class NEXUSIC_REST():
         uri += '?hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(uri, logger=self.logger, logLvl=logging.DEBUG)
+                utils.print_log(str(body), logger=self.logger, logLvl=logging.DEBUG)
+
             res = requests.put(uri, json=body, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Creating new record in ' + tableDBName + ' error ')
             return result, result_code
@@ -620,12 +673,15 @@ class NEXUSIC_REST():
         :return: (``tuple`` - dict or string, string) - response (json, raw or error string) and response status code
         """
 
-        if self.verbose or verbose:
-            print('Importing repository...')
+        if verbose:
+            utils.print_log('Importing repository...', logger=self.logger, logLvl=logging.INFO)
 
         postblob_uri = self.icweb_uri + '/data/icweb.dll/postBlob' +'?hash=' + self.hash
 
         try:
+            if self.logger != None and self.logger.level > 10:
+                utils.print_log(postblob_uri, logger=self.logger, logLvl=logging.DEBUG)
+
             # Execute post blob
             res = requests.post(postblob_uri, data=binary_data, verify=self.verify)
             result, result_code = self.validate_and_return_response(res, 'Importing repository error ')
@@ -805,7 +861,7 @@ class NEXUSIC_REST():
             result, result_code = self.getTable('Lookup_List', xFilter=x_filter, verbose=verbose)
             if len(result['rows']) == 0:
                 raise Exception('Lookup list ' + lookupListName + ' was not found')
-            elif len(result['rows']) > 0:
+            elif len(result['rows']) > 1:
                 raise Exception('Lookup list ' + lookupListName + ' found more than one time')
             else:
                 LL_ID = result['rows'][0]['LL_ID']
@@ -879,7 +935,8 @@ class NEXUSIC_REST():
         prev_survey_set = None
 
         for event_idx, event in enumerate(events):
-            print('Importing new events...' + str(event_idx + 1) + ' of ' + str(len(events)))
+            utils.print_log('Importing new events...' + str(event_idx + 1) + ' of ' + str(len(events)),
+                            logger=self.logger, logLvl=logging.INFO)
 
             # Get Asset by Full Location
             # --------------------------
@@ -1083,8 +1140,10 @@ class NEXUSIC_REST():
                                                                 multimedia['filename'], multimedia['binary_data'],
                                                                 verbose=verbose)
 
-        print('Imported ' + str(len(imported_events)) + ' new events...')
-        print('Missed ' + str(len(missed_events)) + ' events from input...')
+        utils.print_log('Imported ' + str(len(imported_events)) + ' new events...',
+                        logger=self.logger, logLvl=logging.INFO)
+        utils.print_log('Missed ' + str(len(missed_events)) + ' events from input...',
+                        logger=self.logger, logLvl=logging.INFO)
         return imported_events, missed_events
 
     ######################################################################################
@@ -1109,7 +1168,7 @@ class NEXUSIC_REST():
         :param pageSize: (``int`` - optional) - Page size for response (default value None). When default is used the default NEXUS IC page size will be used (100).
         :param verbose: (``bool`` - optional) - Print internal messages if True (default value False).
 
-        :return: (``tuple`` - list, list) - List of imported events and missed events.
+        :return: (``tuple`` - list, list) - List of imported assets and missed assets.
         """
 
         imported_assets = []
@@ -1221,8 +1280,10 @@ class NEXUSIC_REST():
                                      'Asset View': asset['asset_view']}
                 imported_assets.append(import_asset_info)
 
-        print('Imported ' + str(len(imported_assets)) + ' new assets...')
-        print('Missed ' + str(len(missed_assets)) + ' assets from input...')
+        utils.print_log('Imported ' + str(len(imported_assets)) + ' new assets...',
+                        logger=self.logger, logLvl=logging.INFO)
+        utils.print_log('Missed ' + str(len(missed_assets)) + ' assets from input...',
+                        logger=self.logger, logLvl=logging.INFO)
         return imported_assets, missed_assets
 
     ######################################################################################
@@ -1344,11 +1405,13 @@ class NEXUSIC_REST():
         return result, result_code
 
     ######################################################################################
-    def get_rt_id(self, report_name):
+    def get_rt_id(self, report_name, verbose=False):
         """
-        Gets the RT_ID value from the Report_Template table using a report's Name.
-        :param report_name: (``string``) - Name of the Report as it appears in NEXUS.
-        :return: (``int``) - RT_ID value.
+        Gets the RT_ID value from the Report_Template table using a report's Name
+        :param report_name: (``string``) - Name of the Report as it appears in NEXUS
+        :param verbose: (``bool`` - optional) - Print internal messages if True (default value False)
+
+        :return: (``int``) - RT_ID value
         """
 
         xFilter = '{"where": [{"field": "Name", "value": "' + report_name + '"}]}'
@@ -1358,7 +1421,9 @@ class NEXUSIC_REST():
             return str(report_status) + ': ' + str(report_json)
         else:
             rt_id = report_json['rows'][0]['RT_ID']
-            print("RT_ID="+str(rt_id))
+            if verbose:
+                utils.print_log('RT_ID=' + str(rt_id), logger=self.logger, logLvl=logging.INFO)
+
             return rt_id
 
 ######################################################################################
