@@ -1,8 +1,10 @@
 import requests
 import json
+import logging
 from requests.auth import HTTPBasicAuth
 #from requests_kerberos import HTTPKerberosAuth
 from requests_ntlm import HttpNtlmAuth
+from pynexusic import utils
 
 ############################# OSI PI Web API Class ############################
 class OSIPI_WebAPI():
@@ -22,7 +24,8 @@ class OSIPI_WebAPI():
 
     def __init__(self, piwebapi_url, username, password, authentication_type='basic',
                  max_attempts=1, timeout=None,
-                 verbose=False, verify=True):
+                 verbose=False, verify=True,
+                 logger=None):
         """
 
             :param piwebapi_url: (``string``) - OSI PI base URL.
@@ -45,6 +48,7 @@ class OSIPI_WebAPI():
         self.timeout = timeout  # TODO: Functionality to be added
         self.verbose = verbose
         self.verify = verify
+        self.logger = logger
 
         self.authentication_type = self.authenticate(self.authentication_type, self.username, self.password)
         self.sys_links = self.get_system_links()
@@ -63,8 +67,8 @@ class OSIPI_WebAPI():
         """
         assert authentication_type.lower() in ['basic', 'ntlm'], authentication_type + ' authentication type not supported'
 
-        if self.verbose or verbose:
-            print('Authenticating with OSI PI...')
+        if verbose:
+            utils.print_log('Authenticating with OSI PI...', logger=self.logger, logLvl=logging.INFO)
 
         if authentication_type.lower() == 'basic':
             security_auth = HTTPBasicAuth(username, password)
@@ -134,8 +138,8 @@ class OSIPI_WebAPI():
                 **<serverType>** is either DataServers or AssetServers
         """
 
-        if self.verbose or verbose:
-            print('Getting OSI PI system links...')
+        if verbose:
+            utils.print_log('Getting OSI PI system links...', logger=self.logger, logLvl=logging.INFO)
 
         result = {}
         links, links_status = self.webapi_get_request(self.piwebapi_url, pageSize=pageSize)
@@ -166,8 +170,8 @@ class OSIPI_WebAPI():
                     ([{'ServerName': 'xxx', 'Points': 'xxx', 'ResponseStatus': 'xxx'}], None)
         """
 
-        if self.verbose or verbose:
-            print('Getting OSI PI points list...')
+        if verbose:
+            utils.print_log('Getting OSI PI points list...', logger=self.logger, logLvl=logging.INFO)
 
         result = []
         for dataServer in self.sys_links['DataServers']:
@@ -214,8 +218,8 @@ class OSIPI_WebAPI():
         assert searchType.lower() in ['end', 'interpolated', 'recorded',
                                       'plot', 'summary', 'value'], searchType + ' search type not supported'
 
-        if self.verbose or verbose:
-            print('Getting OSI PI stream data...')
+        if verbose:
+            utils.print_log('Getting OSI PI stream data...', logger=self.logger, logLvl=logging.INFO)
 
         if parameters == None:
             url = self.piwebapi_url + r'/streams/' + str(webID) + r'/' + searchType.lower()
@@ -287,8 +291,8 @@ class OSIPI_WebAPI():
             :return: (``tuple`` - dict or string, string) - response (json or error string) and response status code.
         """
 
-        if self.verbose or verbose:
-            print('Getting OSI PI point attributes...')
+        if verbose:
+            utils.print_log('Getting OSI PI point attributes...', logger=self.logger, logLvl=logging.INFO)
 
         url = self.piwebapi_url + r'/points/' + str(webID) + r'/attributes'
         resp, resp_status = self.webapi_get_request(url, pageSize=pageSize)
